%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%               TWO RS METHODS                     %%%%%%%%%%%%
%%%%        FOR SAMPLING A NAKAGAMI PDF               %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all
close all
clc
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(['%%       TWO RS METHODS         %%'])
disp(['%% FOR SAMPLING A NAKAGAMI PDF  %%'])
disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(' ')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% NAKAGAMI target pdf %%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
m=1.8;
K=5;
disp(['Target parameter m = ',num2str(m)])
disp(['Target parameter K = ',num2str(K)])
disp(' ')
Nak_Tar=@(x) x.^(2*m-1).*exp(-(m/K)*x.^2).*(x>0);
%%%% MODE POSITION AND VALUE OF THE NAKAGAMI TARGET
mode_pos=(2^(1/2)*K^(1/2)*(2*m - 1)^(1/2))/(2*m^(1/2));
fval_mode=Nak_Tar(mode_pos);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N_samples_gen_by_eachProposal=20000; %%% number of desired samples
disp(['Number of samples generated by each proposal pdf = ',num2str(N_samples_gen_by_eachProposal)])
disp(' ')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% RS METHOD 1              %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% construction proposal 1  %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% PARAMETER OF THE PROPOSAL: a1, mpr1  
a1=fval_mode;
mpr1=m/K;
%%%%%%% proposal function 1 %%%%%
Prop_fun_1=@(x) a1*exp(-mpr1*(x-mode_pos).^2);
%%% GENERATION from Proposal 1%%%%
xpr1=Nakagami_Gauss_proposal_1(mpr1,mode_pos,N_samples_gen_by_eachProposal);
%%%%% %%%%%%% %%%%%% %%%%%% %%%%%% 
%%% RS TEST - for RS method 1  %%%
pa_RS=Nak_Tar(xpr1)./Prop_fun_1(xpr1); %%% acceptance probability Method 1 
u=rand(1,N_samples_gen_by_eachProposal);
pos_accepted_samples=find(u<=pa_RS);
x_acc1=xpr1(pos_accepted_samples); %%% accepted samples
ACC_RATE_1=length(x_acc1)/N_samples_gen_by_eachProposal;
%%%
disp(['Number of accepted samples with Method 1 = ',num2str(length(x_acc1))])
disp(['Acceptance Rate of Method 1 in this run : ',num2str(ACC_RATE_1)])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% RS METHOD 2              %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% construction proposal 2  %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% PARAMETER OF THE PROPOSAL: a2, mpr2 and Kpr2  
 m_integer=round(m);
 if m_integer-m>0
     mpr2=m_integer-0.5;
 else
     mpr2=m_integer;
 end
Kpr2=(2*mpr2*mode_pos^2)/(2*mpr2 - 1);
fval_mode_proposal=mode_pos.^(2*mpr2-1).*exp(-(mpr2/Kpr2)*mode_pos.^2);
a2=(fval_mode/fval_mode_proposal);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% proposal function 2 %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Prop_fun_2=@(x) a2*x.^(2*mpr2-1).*exp(-(mpr2/Kpr2)*x.^2).*(x>0);
%%% GENERATION from Proposal 2%%%%
xpr2=Nakagami_proposal_2(mpr2,Kpr2,N_samples_gen_by_eachProposal);
%%%%% %%%%%%% %%%%%% %%%%%% %%%%%% 
%%% RS TEST - for RS method 2  %%%
pa_RS=Nak_Tar(xpr2)./Prop_fun_2(xpr2); %%% acceptance probability Method 1 
u=rand(1,N_samples_gen_by_eachProposal);
pos_accepted_samples=find(u<=pa_RS);
x_acc2=xpr2(pos_accepted_samples); %%% accepted samples
ACC_RATE_2=length(x_acc2)/N_samples_gen_by_eachProposal;
%%%
disp(' ')
disp(['Number of accepted samples with Method 2 = ',num2str(length(x_acc2))])
disp(['Acceptance Rate of Method 2 in this run : ',num2str(ACC_RATE_2)])
%%%%%%%%%%%%%
%%% PLOTS %%%
%%%%%%%%%%%%%
h3=figure;
scrsz = get(0,'ScreenSize'); 
set(h3,'OuterPosition',[scrsz(1)+400,scrsz(2)+200,600,750])
subplot(3,1,1)
x=-5:0.001:10;
plot(x,Nak_Tar(x),'k','LineWidth',1.5)
hold on
x=-5:0.04:10;
plot(x,Prop_fun_1(x),'bo','LineWidth',1,'MarkerSize',4.5)
x=-5:0.001:10;
plot(x,Prop_fun_2(x),'r--','LineWidth',1.5)
set(gca,'Fontsize',18)
set(gca,'Fontweight','Bold')
legend('Nakagami Target','Proposal 1','Proposal 2')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
subplot(3,1,2)
    hold on
    [e,b]=hist(x_acc1,30);
    Zbar=sum(e*(b(2)-b(1)));
    hold on
    set(gca,'FontWeight','Bold','FontSize',20)
    box on
    bar(b,1/Zbar*e,'r')
    step1=0.001;
    x1=-5:0.001:10;
    Z=sum(Nak_Tar(x1)*step1);
    plot(x1,(1/Z)*Nak_Tar(x1),'k','LineWidth',4)
    legend('Hist. Method 1','TARGET pdf')
%%%%%%%%%%%%%   
subplot(3,1,3)
    hold on
    [e,b]=hist(x_acc2,30);
    Zbar=sum(e*(b(2)-b(1)));
    hold on
    set(gca,'FontWeight','Bold','FontSize',20)
    box on
    bar(b,1/Zbar*e,'g')
    plot(x1,(1/Z)*Nak_Tar(x1),'k','LineWidth',4)
    legend('Hist. Method 2','TARGET pdf')
   
